@@ -1,4 +1,6 @@
 from os import getcwd
 from gitlab_ps_utils.logger import myLogger
+from gitlab_evaluate.lib.log_utils import get_log_level
 
-log = myLogger(__name__, app_path=getcwd(), log_dir='.', log_name='evaluate')
\ No newline at end of file
+log = myLogger(__name__, app_path=getcwd(), log_dir='.', log_name='evaluate')
+log.setLevel(get_log_level())
\ No newline at end of file
