@@ -4,8 +4,8 @@ from  typing import Optional
 @dataclass
 class User():
    username: str
-   state: str
    email: Optional[str]
+   state: Optional[str]
    using_license_seat: Optional[bool]
    is_admin: bool = False
 
