@@ -23,7 +23,7 @@ def append_to_workbook(worksheet, data, headers):
     row = len(worksheet.table)
     for d in data:
         for col, h in enumerate(headers):
-            write_to_worksheet(worksheet, row, col, d.get(h, ''))
+            write_to_worksheet(worksheet, row, col, d.get(h, d.get(h.lower(), '')))
 
 
 def write_to_worksheet(worksheet, row, col, item):
@@ -37,17 +37,17 @@ def write_to_worksheet(worksheet, row, col, item):
 
 def check_size(k, v):
     # TODO: Dictionary of function pointers
-    if k in ["storageSize", "lfsObjectsSize", "uploadsSize", "buildArtifactsSize"]:
+    if k in ["storage_size", "lfs_objects_size", "uploads_size", "build_artifacts_size", "wiki_size"]:
         return check_storage_size(v)
-    if k == "snippetsSize":
+    if k == "snippets_size":
         return check_num_snippets(v)
-    if k == "commitCount":
+    if k == "commit_count":
         return check_num_commits(v)
-    if k == "repositorySize":
+    if k == "repository_size":
         return check_file_size(v)
-    if k == "Estimated Export Size":
+    if k == "estimated_export_size":
         return check_export_size_5(v)
-    if k == "Estimated Export Size S3":
+    if k == "estimated_export_size_s3":
         return check_export_size_10(v)
 
 
@@ -130,6 +130,45 @@ def to_camel_case(s):
     s = sub(r"(_|-)+", " ", s).title().replace(" ", "")
     return ''.join([s[0].lower(), s[1:]])
 
+def to_snake_case(s):
+    """
+        Shameless copy from https://www.w3resource.com/python-exercises/string/python-data-type-string-exercise-98.php
+    """
+    return '_'.join(
+        sub('([A-Z][a-z]+)', r' \1',
+            sub('([A-Z]+)', r' \1',
+                s.replace('-', ' '))).split()).lower()
+
+def nested_snake_case(dict_obj):
+
+    """
+    Recursively converts all keys in a nested dictionary from camelCase to snake_case.
+    
+    Args:
+        dict_obj: Dictionary object to convert keys for
+        
+    Returns:
+        Dictionary with all keys converted to snake_case
+    """
+    if not isinstance(dict_obj, dict):
+        return dict_obj
+    
+    result = {}
+    for key, value in dict_obj.items():
+        # Convert the key to snake_case
+        snake_key = to_snake_case(key)
+        
+        # Recursively process nested dictionaries
+        if isinstance(value, dict):
+            result[snake_key] = nested_snake_case(value)
+        elif isinstance(value, list):
+            # Handle lists that might contain dictionaries
+            result[snake_key] = [nested_snake_case(item) if isinstance(item, dict) else item for item in value]
+        else:
+            result[snake_key] = value
+    
+    return result
+
 
 def get_date_run():
     return datetime.datetime.now().date().strftime("%Y-%m-%d")
@@ -193,7 +232,37 @@ def get_package_version():
     return importlib.metadata.version("gitlab-evaluate")
 
 
-def sqlite_connection(db):
+def sqlite_connection(db, rows_as_dicts=False):
     con = sqlite3.connect(db)
+    if rows_as_dicts:
+        con.row_factory = sqlite3.Row
     cur = con.cursor()
     return con, cur
+
+def strtobool (val):
+    """Convert a string representation of truth to true (1) or false (0).
+
+    True values are 'y', 'yes', 't', 'true', 'on', and '1'; false values
+    are 'n', 'no', 'f', 'false', 'off', and '0'.  Raises ValueError if
+    'val' is anything else.
+
+    - from distutils
+    """
+    val = val.lower()
+    if val in ('y', 'yes', 't', 'true', 'on', '1'):
+        return True
+    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
+        return False
+    else:
+        raise ValueError("invalid truth value %r" % (val,))
+
+def set_ssl_verification(val):
+    with open('SSL_VERIFICATION', 'w') as f:
+        f.write(str(val))
+
+def get_ssl_verification():
+    try:
+        with open('SSL_VERIFICATION', 'r') as f:
+            return strtobool(f.read())
+    except FileNotFoundError:
+        return True
