@@ -1,7 +1,9 @@
+import json
 import re
 from sys import exit as sys_exit
 import xlsxwriter
 from dateutil import parser
+import time
 
 from gitlab_evaluate.lib import utils
 from gitlab_evaluate.migration_readiness.ado.evaluate import AdoEvaluateClient
@@ -9,9 +11,7 @@ from gitlab_ps_utils.processes import MultiProcessing
 
 
 class AdoReportGenerator:
-
-    def __init__(self, host, token, filename=None, output_to_screen=False, project=None, processes=None, api_version=None, skip_details=False):
-
+    def __init__(self, host, token, filename, output_to_screen=False, project=None, processes=None, api_version=None, skip_details=False, verify=True):
         self.skip_details = skip_details
         self.total_repositories = 0
         self.total_disabled_repositories = 0
@@ -20,15 +20,13 @@ class AdoReportGenerator:
         self.total_projects = 0
         self.total_users = 0
         self.total_agent_pools = 0
+        self.total_feeds = 0
         self.total_variables = 0
         self.total_wikis = 0
         self.host = host
-        self.ado_client = AdoEvaluateClient(host, token, api_version=api_version)
+        self.ado_client = AdoEvaluateClient(host, token, api_version=api_version, verify=verify)
         self.validate_token()
-        if filename:
-            self.workbook = xlsxwriter.Workbook(f'{filename}.xlsx')
-        else:
-            self.workbook = xlsxwriter.Workbook('ado_evaluate_report')
+        self.workbook = xlsxwriter.Workbook(filename)
         self.app_stats = self.workbook.add_worksheet('Organization Insights')
         self.align_left = self.workbook.add_format({'align': 'left'})
         self.header_format = self.workbook.add_format({'bg_color': 'black', 'font_color': 'white', 'bold': True, 'font_size': 10})
@@ -38,6 +36,7 @@ class AdoReportGenerator:
         self.projects = self.workbook.add_worksheet('Raw Project Data')
         self.pipelines = self.workbook.add_worksheet('Pipelines')
         self.raw_output = self.workbook.add_worksheet('Raw Repository Data')
+        self.feeds = self.workbook.add_worksheet('Feeds')
         self.output_to_screen = output_to_screen
         self.processes = processes
         self.project = project
@@ -106,13 +105,32 @@ class AdoReportGenerator:
             'Pipeline Type',
             'Pipeline URL',
         ]
-
+        
+        self.feeds_headers = [
+            'Scope',
+            'Project ID',
+            'Project Name',
+            'Feed ID',
+            'Feed Name',
+            'Package ID',
+            'Package Name',
+            'Package Type',
+            'Total versions',
+            'Total Downloads',
+            'Publish date',
+            'Last downloaded',
+            'Upstream Source ID',
+            'Upstream Source URL',
+            'Upstream Source Type'
+        ]
+        
         utils.write_headers(0, self.raw_output, self.columns, self.header_format)
         utils.write_headers(0, self.users, self.user_headers, self.header_format)
         utils.write_headers(0, self.agent_pools, self.agent_pool_headers, self.header_format)
         utils.write_headers(0, self.variable_groups, self.variables_groups_headers, self.header_format)
         utils.write_headers(0, self.projects, self.project_headers, self.header_format)
         utils.write_headers(0, self.pipelines, self.pipeline_headers, self.header_format)
+        utils.write_headers(0, self.feeds, self.feeds_headers, self.header_format)
         self.multi = MultiProcessing()
 
     def write_workbook(self):
@@ -177,6 +195,9 @@ class AdoReportGenerator:
 
                     self._process_repos(projects['value'], skip_details)
 
+                    # check if rate limit has been hit
+                    self.ado_client.wait_timer(response.headers, "Projects List")
+
                     # Check for next page
                     if not any(key.lower() == "x-ms-continuationtoken" for key in response.headers):
                         break
@@ -305,6 +326,9 @@ class AdoReportGenerator:
             self.total_users += len(users['value'] if 'value' in users else [])
             print(f"Retrieved {self.total_users} users so far...")
 
+            # check if rate limit has been hit
+            self.ado_client.wait_timer(response.headers, "Users List")
+
             # Check if there's a next page
             if not any(key.lower() == "x-ms-continuationtoken" for key in response.headers):
                 print(f"Retrieved a total of {self.total_users} users.")
@@ -328,18 +352,165 @@ class AdoReportGenerator:
                     'Is Hosted': pool.get('isHosted', 'N/A'),
                     'Pool Size': pool.get('size', 'N/A'),
                     'Legacy': pool.get('isLegacy', 'N/A'),
-                    'Owner': pool.get('owner', {}).get('displayName', 'N/A')
+                    'Owner': (pool.get('owner') or {}).get('displayName', 'N/A')
                 }
                 utils.append_to_workbook(self.agent_pools, [agent_pool_data], self.agent_pool_headers)
             self.total_agent_pools += len(agent_pools['value'])
             print(f"Retrieved {len(agent_pools['value'])} agent pools so far...")
 
+            # check if rate limit has been hit
+            self.ado_client.wait_timer(response.headers, "Agent Pools List")
+
             # Check if there's a next page
             if not any(key.lower() == "x-ms-continuationtoken" for key in response.headers):
                 break  # No more pages
             # There is page, so get the continuation token for the next page
             params["continuationToken"] = response.headers["X-MS-ContinuationToken"]
 
+    def handle_getting_feeds_data(self):
+        params = {
+            "$top": "100"
+        }
+        print("Fetching Azure Artifact feeds data...")
+        while True:
+            response = self.ado_client.retry_request(self.ado_client.get_feeds, params)
+            if not response or response.status_code != 200:
+                print(f"Failed to fetch feeds: {response.text if response else 'No response'}")
+                break
+            feeds = response.json()
+            for feed in feeds["value"]:
+                if feed.get('project'):
+                    print(f"Processing feed {feed['name']} in project {feed['project']['name']}...")
+                    self._process_feed_packages(feed, feed.get('project', {}).get('id'))
+                else:
+                    print(f"Processing feed {feed['name']} in organization scope...")
+                    self._process_feed_packages(feed, None)
+            
+            # check if rate limit has been hit
+            self.ado_client.wait_timer(response.headers, "Arifacts Feeds")
+
+            # Check if there's a next page
+            if not any(key.lower() == "x-ms-continuationtoken" for key in response.headers):
+                break  # No more pages
+            # There is page, so get the continuation token for the next page
+            params["continuationToken"] = response.headers["X-MS-ContinuationToken"]
+
+    def _process_feed_packages(self, feed, project_id):
+        """Process all packages in a feed and get their metrics"""
+        packages_params = {'$top': 100}
+        
+        while True:
+            packages_response = self.ado_client.retry_request(
+                self.ado_client.get_packages, packages_params, feed['id'], project_id
+            )
+            if not packages_response or packages_response.status_code != 200:
+                print(f"Failed to fetch packages for feed {feed['name']}: {packages_response.text if packages_response else 'No response'}")
+                break
+                
+            packages = packages_response.json()
+            
+            for package in packages.get("value", []):
+                print(f"Processing package {package.get('name', 'Unknown')} in feed {feed['name']}...")
+                
+                # Get package metrics
+                package_metrics = self._get_package_metrics(feed['id'], package, project_id)
+                
+                # Build comprehensive package data
+                package_data = self._build_package_data(feed, package, package_metrics)
+                
+                # Append to workbook
+                utils.append_to_workbook(self.feeds, [package_data], self.feeds_headers)
+
+            # check if rate limit has been hit
+            self.ado_client.wait_timer(packages_response.headers, "Packages List")
+
+            # Check if there's a next page for packages
+            if not any(key.lower() == "x-ms-continuationtoken" for key in packages_response.headers):
+                break  # No more pages
+            # There is a next page, so get the continuation token for the next page
+            packages_params["continuationToken"] = packages_response.headers["X-MS-ContinuationToken"]
+
+    def _get_package_metrics(self, feed_id, package, project_id):
+        """Get package metrics using packagemetricsbatch endpoint"""
+        try:
+            # Prepare metrics request payload
+            metrics_payload = json.dumps({
+                "packageIds": [package.get('id')]
+            })
+            
+            metrics_response = self.ado_client.get_package_metrics_batch(
+                payload=metrics_payload, 
+                feed_id=feed_id, 
+                project_id=project_id
+            )
+            
+            if metrics_response and metrics_response.status_code == 200:
+                metrics_data = metrics_response.json()
+                print(f"Retrieved metrics for package {package.get('name', 'Unknown')}")
+                return metrics_data
+            else:
+                print(f"Failed to get metrics for package {package.get('name', 'Unknown')}: {metrics_response.text if metrics_response else 'No response'}")
+                return None
+                
+        except Exception as e:
+            print(f"Error getting package metrics: {e}")
+            return None
+
+    def _build_package_data(self, feed, package, package_metrics):
+        """Build comprehensive package data including metrics"""
+        
+        total_downloads = "N/A"
+        last_download_date = "N/A"
+        publish_date = "N/A"
+        total_versions = "N/A"
+        upstream_source_id = "N/A"
+        upstream_source_url = "N/A"
+        upstream_source_type = "N/A"
+        
+        if package_metrics and package_metrics.get('value'):
+            for metric in package_metrics['value']:
+                if metric.get('packageId') == package.get('id'):
+                    total_downloads = metric.get('downloadCount', 0)
+                    last_download_date = self.convert_utc_to_local(metric.get('lastDownloaded', 'N/A'))
+                    break
+        
+        latest_version = None
+        if 'versions' in package:
+            total_versions = len(package['versions'])
+            for version in package.get('versions', []):
+                if version.get('isLatest', False):
+                    latest_version = version
+                    break
+
+            if latest_version:
+                publish_date = self.convert_utc_to_local(latest_version.get('publishDate'))
+                upstream_source_id = latest_version.get('directUpstreamSourceId', 'N/A')
+        
+        if upstream_source_id != "N/A" and feed.get('upstreamSources'):
+            for upstream_source in feed['upstreamSources']:
+                if upstream_source.get('id') == upstream_source_id:
+                    upstream_source_url = upstream_source.get('location', 'N/A')
+                    upstream_source_type = upstream_source.get('upstreamSourceType', 'N/A')
+                    break
+        
+        return {
+            'Scope': 'Project' if feed.get('project') else 'Organization',
+            'Project ID': feed.get('project', {}).get('id', 'N/A') if feed.get('project') else 'N/A',
+            'Project Name': feed.get('project', {}).get('name', 'N/A') if feed.get('project') else 'N/A',
+            'Feed ID': feed.get('id', 'N/A'),
+            'Feed Name': feed.get('name', 'N/A'),
+            'Package ID': package.get('id', 'N/A'),
+            'Package Name': package.get('name', 'N/A'),
+            'Package Type': package.get('protocolType', 'N/A'),
+            'Total versions': total_versions,
+            'Total Downloads': total_downloads,
+            'Publish date': publish_date,
+            'Last downloaded': last_download_date,
+            'Upstream Source ID': upstream_source_id,
+            'Upstream Source URL': upstream_source_url,
+            'Upstream Source Type': upstream_source_type
+        }
+
     def handle_getting_variable_groups_data(self):
 
         if self.project:
@@ -405,19 +576,22 @@ class AdoReportGenerator:
 
             if skip_details is False:
                 branches = []
-                params = {'$top': 1000}
+                branch_params = {'$top': 1000}
                 print(f"Fetching branches for repo {repository_id}...")
                 while True:
-                    branches_response = self.ado_client.retry_request(self.ado_client.get_branches, params, project_id, repository_id)
+                    branches_response = self.ado_client.retry_request(self.ado_client.get_branches, branch_params, project_id, repository_id)
                     if branches_response and branches_response.status_code == 200:
                         branches.extend(branches_response.json()['value'])
                         print(f"Retrieved {len(branches)} branches so far...")
 
+                        # check if rate limit has been hit
+                        self.ado_client.wait_timer(branches_response.headers, "Branches List")
+
                         # Check if there's a next page
                         if not any(key.lower() == "x-ms-continuationtoken" for key in branches_response.headers):
                             break  # No more pages
                         # There is page, so get the continuation token for the next page
-                        params["continuationToken"] = branches_response.headers["X-MS-ContinuationToken"]
+                        branch_params["continuationToken"] = branches_response.headers["X-MS-ContinuationToken"]
                     else:
                         break
             else:
@@ -426,24 +600,29 @@ class AdoReportGenerator:
 
             if skip_details is False:
                 pull_requests = []
+                prs_params = {'$top': 1000}
                 print(f"Fetching pull requests for repo {repository_id}...")
                 while True:
-                    prs_response = self.ado_client.retry_request(self.ado_client.get_prs, params, project_id, repository_id)
+                    prs_response = self.ado_client.retry_request(self.ado_client.get_prs, prs_params, project_id, repository_id)
                     if prs_response and prs_response.status_code == 200:
                         if len(prs_response.json()['value']) > 0:
                             pull_requests.extend(prs_response.json()['value'])
                             print(f"Retrieved {len(pull_requests)} pull requests so far...")
+
                             if "dev.azure.com" in self.host:
                                 # Check if there's a next page
                                 if not any(key.lower() == "x-ms-continuationtoken" for key in prs_response.headers):
                                     break  # No more pages
                                 # There is page, so get the continuation token for the next page
-                                params["continuationToken"] = prs_response.headers["X-MS-ContinuationToken"]
+                                prs_params["continuationToken"] = prs_response.headers["X-MS-ContinuationToken"]
                             else:
-                                params['$skip'] = len(pull_requests)
+                                prs_params['$skip'] = len(pull_requests)
 
                         else:
                             break
+
+                        # check if rate limit has been hit
+                        self.ado_client.wait_timer(prs_response.headers, "Repository Pull Requests")
                     else:
                         break
             else:
@@ -452,16 +631,21 @@ class AdoReportGenerator:
 
             if skip_details is False:
                 commits = []
+                commit_params = {'$top': 1000}
                 print(f"Fetching commits for repo {repository_id}...")
                 while True:
-                    commits_response = self.ado_client.retry_request(self.ado_client.get_commits, params, project_id, repository_id)
+                    commits_response = self.ado_client.retry_request(self.ado_client.get_commits, commit_params, project_id, repository_id)
                     if commits_response and commits_response.status_code == 200:
                         if len(commits_response.json()['value']) > 0:
                             commits.extend(commits_response.json()['value'])
                             print(f"Retrieved {len(commits)} commits so far...")
-                            params['$skip'] = len(commits)
+                            commit_params['$skip'] = len(commits)
                         else:
                             break
+                    
+                        # check if rate limit has been hit
+                        self.ado_client.wait_timer(commits_response.headers, "Repository Commits")
+                
                 last_activity = commits[0]['committer']['date'] if commits else 'N/A'
                 commit_count = len(commits)
             else:
