@@ -1,19 +1,22 @@
 from sys import exit as sys_exit
 import xlsxwriter
 from dacite import from_dict
-from gitlab_ps_utils.json_utils import json_pretty
 from gitlab_ps_utils.processes import MultiProcessing
 from gitlab_ps_utils.misc_utils import is_error_message_present
+from gitlab_ps_utils.dict_utils import dig
 from gitlab_evaluate import log
-from gitlab_evaluate.lib import utils
+from gitlab_evaluate.lib import utils, db_utils
 from gitlab_evaluate.lib.api_models.application_stats import GitLabApplicationStats
-from gitlab_evaluate.migration_readiness.gitlab import evaluate as evaluateApi
+from gitlab_evaluate.lib.api_models.user import User
+from gitlab_evaluate.migration_readiness.gitlab.data_classes.project import Project
+from gitlab_evaluate.migration_readiness.gitlab.evaluate import EvaluateApi
+from gitlab_evaluate.migration_readiness.gitlab import constants
 
 class ReportGenerator():
     def __init__(self, host, token, filename=None, output_to_screen=False, evaluate_api=None, processes=None):
         self.host = host
         self.token = token
-        self.evaluate_api = evaluate_api if evaluate_api else evaluateApi.EvaluateApi()
+        self.evaluate_api = evaluate_api if evaluate_api else EvaluateApi()
         self.validate_token()
         if filename:
             self.workbook = xlsxwriter.Workbook(f'{filename}.xlsx')
@@ -33,93 +36,19 @@ class ReportGenerator():
         self.output_to_screen = output_to_screen
         self.multi = MultiProcessing()
         self.processes = processes
-        self.columns = [
-            'Project',
-            'ID',
-            'URL',
-            'kind',
-            'namespace',
-            'mirror',
-            'archived',
-            'last_activity_at',
-            'Pipelines',
-            'Pipelines_over',
-            'Issues',
-            'Issues_over',
-            'Branches',
-            'Branches_over',
-            'Commits',
-            'Commits_over',
-            'Merge Requests',
-            'Merge Requests_over',
-            'Storage',
-            'storage_size_over',
-            'Repository',
-            'repository_size_over',
-            'wikiSize',
-            "lfsObjectsSize",
-            "lfsObjectsSize_over",
-            "buildArtifactsSize",
-            "buildArtifactsSize_over",
-            "snippetsSize",
-            "snippetsSize_over",
-            "uploadsSize",
-            "uploadsSize_over",
-            'Tags',
-            'Tags_over',
-            'Package Types In Use',
-            'Packages',
-            'packages_size_over',
-            'Containers',
-            'containers_size_over',
-            'Estimated Export Size',
-            'Estimated Export Size Over',
-            'Estimated Export Size S3 Over']
-        self.report_headers = [
-            'Project',
-            'Reason'
-        ]
-        self.user_headers = [
-            'username',
-            'email',
-            'state',
-            'using_license_seat'
-        ]
-        self.account_headers = [
-            'Account',
-            'Comments'
-        ]
-        self.projects_summary_headers = [
-            'Projects',
-            'ALL PROJECTS',
-            'ONLY GROUP PROJECTS',
-            'Comments'
-        ]
-        self.projects_to_review_headers = [
-            'Projects To Review',
-            'ALL PROJECTS',
-            'ONLY GROUP PROJECTS',
-            'Comments'
-        ]
-        self.metrics_headers = [
-            'Metrics',
-            'ALL PROJECTS',
-            'ONLY GROUP PROJECTS',
-            'Comments'
-        ]
         utils.write_headers(0, self.raw_output,
-                            self.columns, self.header_format)
+                            constants.COLUMNS, self.header_format)
         utils.write_headers(0, self.flagged_projects,
-                            self.columns, self.header_format)
+                            constants.COLUMNS, self.header_format)
         utils.write_headers(0, self.final_report,
-                            self.report_headers, self.header_format)
+                            constants.REPORT_HEADERS, self.header_format)
         utils.write_headers(
-            0, self.users, self.user_headers, self.header_format)
+            0, self.users, constants.USER_HEADERS, self.header_format)
         # Merging the first two headers of account summary
         self.app_stats.merge_range(
-            'A1:B1', self.account_headers[0], self.header_format)
+            'A1:B1', constants.ACCOUNT_HEADERS[0], self.header_format)
         self.app_stats.merge_range(
-            'C1:D1', self.account_headers[1], self.header_format)
+            'C1:D1', constants.ACCOUNT_HEADERS[1], self.header_format)
         self.final_report.set_default_row(150)
         self.final_report.set_row(0, 20)
 
@@ -139,29 +68,56 @@ class ReportGenerator():
                 self.host, self.token, f'groups/{group_id}').json()['full_path']
             log.info(
                 f"Running Evaluate on GitLab group '{full_path}' (ID: {group_id})")
-        for flags, messages, results in self.multi.start_multi_process_stream_with_args(self.evaluate_api.get_all_project_data, self.evaluate_api.get_all_projects_by_graphql(
-                self.host, self.token, full_path), self.host, self.token, processes=self.processes):
-            self.write_output_to_files(flags, messages, results)
+        else:
+            log.info(f"Running Evaluate against all projects")
+
+        # Use the wrapper function with SSL configuration instead of bound method
+        self.multi.start_multi_process_stream_with_args(
+            self.evaluate_api.get_all_project_data,
+            self.evaluate_api.get_all_projects_by_graphql(self.host, self.token, full_path),
+            self.host, self.token,
+            processes=self.processes)
+        self.write_output_to_files()
 
     def handle_getting_user_data(self, group_id=None):
-        endpoint = f"groups/{group_id}/members" if group_id else "/users?exclude_internal=true&without_project_bots=true"
+        params = {}
+        endpoint = ''
+        if group_id:
+            endpoint = f"groups/{group_id}/members"
+        else:
+            endpoint = "users"
+            params = {
+                'exclude_internal': True,
+                'without_project_bots': True
+            }
         for user in self.multi.start_multi_process_stream(self.evaluate_api.get_user_data, self.evaluate_api.gitlab_api.list_all(
-                self.host, self.token, endpoint), processes=self.processes):
+                self.host, self.token, endpoint, params=params), processes=self.processes):
+            self.evaluate_api.insert_user_data(user)
+
+        self.write_users_to_report()
+
+    def write_users_to_report(self):
+        log.info("Writing users to report")
+        connection, cursor = utils.sqlite_connection('gitlab.db', rows_as_dicts=True)
+        for data in cursor.execute("SELECT * FROM users"):
+            user = User(**data).to_dict()
             utils.append_to_workbook(
-                self.users, [user.to_dict()], self.user_headers)
+                self.users, [user], constants.USER_HEADERS)
+        connection.close()
 
     def get_app_stats(self, source, token, group_id, admin=True):
         report_stats = []
         additional_info = []
         app_stats = {}
         archived_projects = ""
-        if admin:
+        if admin and not group_id:
             error, resp = is_error_message_present(
                 self.evaluate_api.getApplicationInfo(source, token))
             if not error:
                 app_stats = from_dict(data_class=GitLabApplicationStats, data=resp)
                 archived_projects = self.evaluate_api.getArchivedProjectCount(
                     source, token)
+                self.compare_projects_kind_sum(int(app_stats.projects.replace(',','')))
             else:
                 log.warning(
                     f"Unable to pull application info from URL: {source}")
@@ -223,16 +179,24 @@ class ReportGenerator():
         self.get_projects_summary(project_summary_row_start_index,
                                   app_stats, archived_projects, group_id, source, token)
 
+    def compare_projects_kind_sum(self, projects):
+        group_projects = self.get_project_count_from_db(project_type='group')
+        user_projects = self.get_project_count_from_db(project_type='user')
+
+        if projects != (user_projects + group_projects):
+            if projects != (group_projects + user_projects):
+                log.warning(f"Total Projects ({projects}) does not match the sum of Group ({group_projects}) and User ({user_projects}) Projects")
+
     def get_projects_summary(self, row_start_index, app_stats, archived_projects, group_id, source, token):
         projects_summary = []
         if not app_stats and len(archived_projects) > 0:
             projects_summary += [
                 ('Total', app_stats.projects, utils.get_countif(
                     self.raw_output.get_name(), 'group', 'D')),
-                ('Active', utils.get_countif(self.raw_output.get_name(), 'Fals*', 'F'),
-                 utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Fals*', 'F')),
+                ('Active', utils.get_countif(self.raw_output.get_name(), 'Fals*', 'H'),
+                 utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Fals*', 'H')),
                 ('Archived', archived_projects, utils.get_countifs(
-                    self.raw_output.get_name(), 'group', 'D', 'Tru*', 'F')),
+                    self.raw_output.get_name(), 'group', 'D', 'Tru*', 'H')),
                 ('Outliers', utils.get_if(utils.get_counta(self.flagged_projects.get_name(), 'A')+'=0', 0, utils.get_counta(self.flagged_projects.get_name(), 'A')+'-1'),
                  utils.get_if(utils.get_countif(self.flagged_projects.get_name(), 'group', 'D')+'=0', 0, utils.get_countif(self.flagged_projects.get_name(), 'group', 'D'))),
             ]
@@ -240,15 +204,15 @@ class ReportGenerator():
             projects_summary += [
                 ('Total', self.evaluate_api.get_total_project_count(source, token,
                  group_id), utils.get_countif(self.raw_output.get_name(), 'group', 'D')),
-                ('Active', utils.get_countif(self.raw_output.get_name(), 'Fals*', 'F'),
-                 utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Fals*', 'F')),
+                ('Active', utils.get_countif(self.raw_output.get_name(), 'Fals*', 'H'),
+                 utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Fals*', 'H')),
                 ('Archived', utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*',
-                 'F'), utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'F')),
+                 'H'), utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'H')),
                 ('Outliers', utils.get_if(utils.get_counta(self.flagged_projects.get_name(), 'A')+'=0', 0, utils.get_counta(self.flagged_projects.get_name(), 'A')+'-1'),
                  utils.get_if(utils.get_countif(self.flagged_projects.get_name(), 'group', 'D')+'=0', 0, utils.get_countif(self.flagged_projects.get_name(), 'group', 'D'))),
             ]
         utils.write_headers(row_start_index, self.app_stats,
-                            self.projects_summary_headers, self.header_format)
+                            constants.PROJECT_SUMMARY_HEADERS, self.header_format)
         for row_num, row_data in enumerate(projects_summary):
             for col_num, value in enumerate(row_data):
                 if col_num == 0:
@@ -275,32 +239,32 @@ class ReportGenerator():
             ('Commits > 50,000', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'P'),
              utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'P')),
             ('Merge Requests > 5,000', utils.get_countif(self.raw_output.get_name(), 'Tru*',
-             'Q'), utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'R')),
+             'R'), utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'R')),
             ('Storage Size > 20 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'T'),
              utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'T')),
             ('Repo Size > 5 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'V'),
              utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'V')),
-            ('LFS Objects Size > 5GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'Y'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'Y')),
-            ('Build Artifacts Size > 5GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AA'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AA')),
-            ('Snippets > 1000', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AC'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AC')),
-            ('Uploads Size > 5GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AE'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AE')),
-            ('Tags > 5000', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AG'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AG')),
-            ('Packages Size > 20 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AJ'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AJ')),
-            ('Containers Size > 20 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AL'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AL')),
-            ('Export Size > 5 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AN'),
+            ('LFS Objects Size > 5GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'Z'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'Z')),
+            ('Build Artifacts Size > 5GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AB'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AB')),
+            ('Snippets > 1000', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AD'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AD')),
+            ('Uploads Size > 5GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AF'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AF')),
+            ('Tags > 5000', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AH'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AH')),
+            ('Packages Size > 20 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AK'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AK')),
+            ('Containers Size > 20 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AN'),
              utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AN')),
-            ('Export Size > 10 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AO'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AO')),
+            ('Export Size > 5 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AP'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AP')),
+            ('Export Size > 10 GB', utils.get_countif(self.raw_output.get_name(), 'Tru*', 'AQ'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', 'Tru*', 'AQ')),
         ]
         utils.write_headers(row_start_index, self.app_stats,
-                            self.projects_to_review_headers, self.header_format)
+                            constants.PROJECTS_TO_REVIEW_HEADERS, self.header_format)
         for row_num, row_data in enumerate(projects_to_review):
             for col_num, value in enumerate(row_data):
                 if col_num == 0:
@@ -325,49 +289,53 @@ class ReportGenerator():
              utils.get_sumif(self.raw_output.get_name(), 'D', 'O', 'group')),
             ('Merge Requests', utils.get_sum(self.raw_output.get_name(), 'Q'),
              utils.get_sumif(self.raw_output.get_name(), 'D', 'Q', 'group')),
-            ('Storage Size', utils.get_sum(self.raw_output.get_name(), 'Q'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'Q', 'group')),
+            ('Storage Size', utils.get_sum(self.raw_output.get_name(), 'S'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'S', 'group')),
             ('Repos Size', utils.get_sum(self.raw_output.get_name(), 'U'),
              utils.get_sumif(self.raw_output.get_name(), 'D', 'U', 'group')),
-            ('LFS Objects Size', utils.get_sum(self.raw_output.get_name(), 'X'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'X', 'group')),
-            ('Build Artifacts Size', utils.get_sum(self.raw_output.get_name(), 'Z'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'Z', 'group')),
-            ('Snippets', utils.get_sum(self.raw_output.get_name(), 'AB'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'AB', 'group')),
-            ('Uploads Size', utils.get_sum(self.raw_output.get_name(), 'AD'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'AD', 'group')),
-            ('Tags', utils.get_sum(self.raw_output.get_name(), 'AF'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'AF', 'group')),
-            ('generic Packages', utils.get_countif(self.raw_output.get_name(), '*generic*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*generic*', 'AH')),
-            ('maven Packages', utils.get_countif(self.raw_output.get_name(), '*maven*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*maven*', 'AH')),
-            ('npm Packages', utils.get_countif(self.raw_output.get_name(), '*npm*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*npm*', 'AH')),
-            ('pypi Packages', utils.get_countif(self.raw_output.get_name(), '*pypi*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*pypi*', 'AH')),
-            ('helm Packages', utils.get_countif(self.raw_output.get_name(), '*helm*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*helm*', 'AH')),
-            ('composer Packages', utils.get_countif(self.raw_output.get_name(), '*composer*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*composer*', 'AH')),
-            ('nuget Packages', utils.get_countif(self.raw_output.get_name(), '*nuget*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*nuget*', 'AH')),
-            ('conan Packages', utils.get_countif(self.raw_output.get_name(), '*conan*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*conan*', 'AH')),
-            ('golang Packages', utils.get_countif(self.raw_output.get_name(), '*golang*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*golang*', 'AH')),
-            ('terraform_module Packages', utils.get_countif(self.raw_output.get_name(), '*terraform_module*', 'AH'),
-             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*terraform_module*', 'AH')),
-            ('Packages Size', utils.get_sum(self.raw_output.get_name(), 'AI'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'AI', 'group')),
-            ('Containers Size', utils.get_sum(self.raw_output.get_name(), 'AK'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'AK', 'group')),
-            ('Exports Size', utils.get_sum(self.raw_output.get_name(), 'AM'),
-             utils.get_sumif(self.raw_output.get_name(), 'D', 'AM', 'group'))
+             ('Wiki Size', utils.get_sum(self.raw_output.get_name(), 'W'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'W', 'group')),
+            ('LFS Objects Size', utils.get_sum(self.raw_output.get_name(), 'Y'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'Y', 'group')),
+            ('Build Artifacts Size', utils.get_sum(self.raw_output.get_name(), 'AA'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AA', 'group')),
+            ('Snippets', utils.get_sum(self.raw_output.get_name(), 'AC'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AC', 'group')),
+            ('Uploads Size', utils.get_sum(self.raw_output.get_name(), 'AE'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AE', 'group')),
+            ('Tags', utils.get_sum(self.raw_output.get_name(), 'AG'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AG', 'group')),
+            ('generic Packages', utils.get_countif(self.raw_output.get_name(), '*generic*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*generic*', 'AI')),
+            ('maven Packages', utils.get_countif(self.raw_output.get_name(), '*maven*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*maven*', 'AI')),
+            ('npm Packages', utils.get_countif(self.raw_output.get_name(), '*npm*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*npm*', 'AI')),
+            ('pypi Packages', utils.get_countif(self.raw_output.get_name(), '*pypi*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*pypi*', 'AI')),
+            ('helm Packages', utils.get_countif(self.raw_output.get_name(), '*helm*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*helm*', 'AI')),
+            ('composer Packages', utils.get_countif(self.raw_output.get_name(), '*composer*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*composer*', 'AI')),
+            ('nuget Packages', utils.get_countif(self.raw_output.get_name(), '*nuget*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*nuget*', 'AI')),
+            ('conan Packages', utils.get_countif(self.raw_output.get_name(), '*conan*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*conan*', 'AI')),
+            ('golang Packages', utils.get_countif(self.raw_output.get_name(), '*golang*', 'AI'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*golang*', 'AI')),
+            ('terraform_module Packages', utils.get_countif(self.raw_output.get_name(), '*terraform_module*', 'IH'),
+             utils.get_countifs(self.raw_output.get_name(), 'group', 'D', '*terraform_module*', 'AI')),
+            ('Packages Size', utils.get_sum(self.raw_output.get_name(), 'AJ'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AJ', 'group')),
+            ('Container Tags', utils.get_sum(self.raw_output.get_name(), 'AL'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AL', 'group')),
+            ('Containers Size', utils.get_sum(self.raw_output.get_name(), 'AM'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AM', 'group')),
+            ('Exports Size', utils.get_sum(self.raw_output.get_name(), 'AO'),
+             utils.get_sumif(self.raw_output.get_name(), 'D', 'AO', 'group'))
         ]
         utils.write_headers(row_start_index, self.app_stats,
-                            self.metrics_headers, self.header_format)
+                            constants.METRICS_HEADERS, self.header_format)
         for row_num, row_data in enumerate(metrics):
             for col_num, value in enumerate(row_data):
                 if col_num == 0:
@@ -377,21 +345,20 @@ class ReportGenerator():
                     self.app_stats.write_formula(
                         row_num+row_start_index+1, col_num, '=' + str(value) if value is not None else '')
 
-    def write_output_to_files(self, flags, messages, results):
-        dict_data = []
-        dict_data.append({x: results.get(x) for x in self.columns})
-        utils.append_to_workbook(self.raw_output, dict_data, self.columns)
-
-        if True in flags:
-            utils.append_to_workbook(
-                self.flagged_projects, dict_data, self.columns)
-            utils.append_to_workbook(self.final_report, [{'Project': results.get(
-                'Project'), 'Reason': messages.generate_report_entry()}], self.report_headers)
-        if self.output_to_screen:
-            print(f"""
-            {'+' * 40}
-            {json_pretty(results)}
-            """)
+    def write_output_to_files(self):
+        connection, cursor = utils.sqlite_connection('gitlab.db', rows_as_dicts=True)
+        log.info("Writing raw_project_data sheet")
+        for data in cursor.execute("SELECT * FROM projects"):
+            project = db_utils.convert_retain_bool(Project, data)
+            utils.append_to_workbook(self.raw_output, [project.to_dict()], constants.COLUMNS)
+        log.info("Writing flagged_projects sheet")
+        for data in cursor.execute("SELECT * FROM flagged_projects"):
+            project = db_utils.convert_retain_bool(Project, data)
+            utils.append_to_workbook(self.flagged_projects, [project.to_dict()], constants.COLUMNS)
+        log.info("Writing evaluate_report sheet")
+        for data in cursor.execute("SELECT * FROM reports"):
+            utils.append_to_workbook(self.final_report, [{'project': data['project'], 'reason': data['reason']}], constants.REPORT_HEADERS)
+        connection.close()
 
     def validate_token(self):
         error, resp = is_error_message_present(
@@ -411,27 +378,49 @@ class ReportGenerator():
         '''
             Gets stats of a specific group when using a non-admin token
 
-            Some of the data we normally pull in as an admin canot be retrieved because 
-            the nest API calls available to retrieve that data will add a significant 
+            Some of the data we normally pull in as an admin canot be retrieved because
+            the nest API calls available to retrieve that data will add a significant
             amount of time to retrieve those counts
         '''
-        members = self.evaluate_api.gitlab_api.get_total_count(self.host, self.token, f'groups/{group_id}/members/all')
-        sub_groups = self.evaluate_api.gitlab_api.get_total_count(self.host, self.token, f'groups/{group_id}/descendant_groups')
-        projects = self.evaluate_api.gitlab_api.get_total_count(self.host, self.token, f'groups/{group_id}/projects?include_subgroups=true', params={'include_subgroups': "true"})
-        issues = self.evaluate_api.gitlab_api.get_total_count(self.host, self.token, f'groups/{group_id}/issues')
-        merge_requests = self.evaluate_api.gitlab_api.get_total_count(self.host, self.token, f'groups/{group_id}/merge_requests')
         # Placeholder message for data we cannot currently retrieve with an owner token
         missing_msg = 'unable to retrieve total count'
+        if group_stats := self.evaluate_api.get_specific_group_stats_response(self.host, self.token, group_id):
+            # Return actual results
+            return GitLabApplicationStats(
+                users=dig(group_stats, 'data', 'group', 'groupMembersCount'),
+                active_users=dig(group_stats, 'data', 'group', 'groupMembersCount'),
+                groups=dig(group_stats, 'data', 'group', 'descendantGroups', 'count'),
+                projects=dig(group_stats, 'data', 'group', 'projects', 'count'),
+                issues=dig(group_stats, 'data', 'group', 'issues', 'count'),
+                merge_requests=dig(group_stats, 'data', 'group', 'mergeRequests', 'count'),
+                notes=missing_msg,
+                milestones=missing_msg,
+                forks=missing_msg,
+                ssh_keys=missing_msg,
+                snippets=missing_msg
+            )
+        # Return fallback results with missing message for data that we cannot retrieve
+        log.error("Could not retrieve specific group stats from GitLab")
         return GitLabApplicationStats(
-            users=members,
-            active_users=members,
-            groups=sub_groups,
-            projects=projects,
-            issues=issues,
-            merge_requests=merge_requests,
-            notes=missing_msg,
-            milestones=missing_msg,
-            forks=missing_msg,
-            ssh_keys=missing_msg,
-            snippets=missing_msg
-        )
+                users=missing_msg,
+                active_users=missing_msg,
+                groups=missing_msg,
+                projects=missing_msg,
+                issues=missing_msg,
+                merge_requests=missing_msg,
+                notes=missing_msg,
+                milestones=missing_msg,
+                forks=missing_msg,
+                ssh_keys=missing_msg,
+                snippets=missing_msg
+            )
+
+    def get_project_count_from_db(self, project_type=None):
+        connection, cursor = utils.sqlite_connection('gitlab.db')
+        where_clause = f"WHERE kind = '{project_type}'" if project_type else ""
+        projects = cursor.execute(f"SELECT COUNT(*) FROM projects {where_clause}").fetchone()[-1]
+        connection.close()
+        if isinstance(projects, str):
+            return int(projects.replace(',',''))
+        return int(projects)
+
