@@ -5,8 +5,8 @@ class Job():
     _class: str
     name: str
     url: str
-    color: str
-    fullName: str
+    color: str = "notbuilt"
+    fullName: str = ""
 
     def to_dict(self):
         return asdict(self)
